/***************************************************************************
                          apollon.cpp -  description
                             -------------------
begin		: Dec 05 2003
copyright	: (C) 2002 by Bastiaan Naber, (C) 2002 by Robert Wittams
email		: bastiaan@ricardis.tudelft.nl
description	: 

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>
#include <signal.h>

#include <kcmdlineargs.h>
#include <kconfig.h>
#include <kiconloader.h>
#include <kjanuswidget.h>
#include <klocale.h>
#include <kmenubar.h>
#include <kpopupmenu.h>
#include <kstatusbar.h>
#include <kstdaccel.h>
#include <kstdaction.h>
#include <kprocess.h>

#include <qapplication.h>
#include <qlabel.h>
#include <qstring.h>
#include <qstringlist.h>
#include <qvbox.h>

#include <libgift/libgift.h>
#include <libgift/conf.h>

#include "apollon.h"
#include "apolloninfotab.h"
#include "apollonplayertab.h"
#include "apollonsearchtab.h"
#include "apollonsharedviewtab.h"
#include "apollonsystemtray.h"
#include "apollonsystemtraygnome.h"
#include "apollontransfertab.h"
#include "giftconnection.h"
#include "preferences.h"
#include "sigdialog.h"

ApollonApp::ApollonApp(QWidget *parent, const char *name) :
  KMainWindow(parent, name),
  m_shutdown(false),
  m_killGiFT(true),
  m_startDocked(false)
{
	QString  host;
	int      port;
	bool     remote;

	m_gcn = new giFTConnection();

	m_settingsDialog = new ApollonPreferencesDialog();
	m_sigDialog = new SigDialog(m_gcn);

	m_statsLabel = new QLabel(this);
	m_statsLabel->setFrameStyle(QFrame::Panel | QFrame::Sunken);
	m_statsLabel->setAlignment(AlignBottom | AlignRight);

	connect(m_gcn, SIGNAL(giftShareReply(QString)), this, SLOT(giftShareReply(QString)));
	connect(m_gcn, SIGNAL(sigGiFTPID(pid_t)),       this, SLOT(setGiFTpid(pid_t)));

	KCmdLineArgs *args = KCmdLineArgs::parsedArgs();

	QString commandlineServer = args->getOption("s");

	if(!commandlineServer)
	{
		QString user(getenv("USER"));
		QString home(getenv("HOME"));
		QString filename(home);
		filename.append("/.giFT/ui/ui.conf");

		Config *config = config_new_ex((char *) filename.ascii(), false);

		host = config_get_str(config, (char *) "daemon/host");
		port = config_get_int(config, (char *) "daemon/port");
		remote = false;

		config_free(config);
	}
	else
	{
		QStringList PortHostList = QStringList::split(":", commandlineServer);
		host = PortHostList[0];
		port = PortHostList[1].toInt();
		remote = true;
	}

	if(args->isSet("d")) // apollon -d enables debug output
	{
		m_gcn->setDebugOutput(args->isSet("d")); // here you can add other objects which might have debug-output
	}

	m_gcn->setParams(host, port, remote);
	m_gcn->connectToServer();

	initStatusBar();
	initActions();
	initCentre();
	initSysTray();

	statsUpdate(0, 0.0);

	readConfig();
}

bool ApollonApp::queryClose()
{
	if(m_shutdown)
	{
		return true;
	}

	hide();
	return false;
}

void ApollonApp::quit()
{
	qApp->quit();
	delete this;
}

ApollonApp::~ApollonApp()
{
	if(m_killGiFT)
	{
		// kill(giFT_pid, 9);
		system("killall giftd");
	}
}

void ApollonApp::initStatusBar()
{
	statusBar()->message(i18n("Connecting..."));
	m_statsLabel->setText("                      ");
	statusBar()->addWidget(m_statsLabel, 0, true);
}

void ApollonApp::statsUpdate(int activeDownloads, float rate)
{
	QString stats;
	stats = i18n("1 active download at %1 KB/s",
	             "%n active downloads at %1 KB/s", activeDownloads).arg(rate);
	m_statsLabel->setText(stats);
}

void ApollonApp::readConfig()
{
	setAutoSaveSettings();
	m_settingsDialog->readConfig();
	setAutoSaveSettings("MainWindow");
}

void ApollonApp::saveConfig()
{
	m_settingsDialog->saveConfig();
}

void ApollonApp::initActions()
{
	KAction *fileExitAction =        KStdAction::quit(                              this,           SLOT(quit()),       actionCollection(), "fileExitAction");
	KAction *sharesHideAction =      new KAction(i18n("&Hide Shares"),           0, this,           SLOT(sharesHide()), actionCollection(), "sharesHideAction");
	KAction *sharesShowAction =      new KAction(i18n("&Show Shares"),           0, this,           SLOT(sharesShow()), actionCollection(), "sharesShowAction");
	KAction *sharesSyncAction =      new KAction(i18n("S&ync"),        "reload", 0, this,           SLOT(sharesSync()), actionCollection(), "sharesSyncAction");
	KAction *settingsOptionsAction = KStdAction::preferences(                       m_settingsDialog, SLOT(exec()),       actionCollection(), "settingsOptionsAction");
	KAction *sigAddlinkAction =      new KAction(i18n("&Add Sig2Dat link"), "bookmark_add", 0, m_sigDialog, SLOT(exec()), actionCollection(), "sigAddlinkAction");

	// create File menu
	KPopupMenu *fileMenu = new KPopupMenu(this);
	sigAddlinkAction->plug(fileMenu);
	fileMenu->insertSeparator();
	fileExitAction->plug(fileMenu);

	// create Shares menu
	KPopupMenu *sharesMenu = new KPopupMenu(this);
	sharesHideAction->plug(sharesMenu);
	sharesShowAction->plug(sharesMenu);
	sharesMenu->insertSeparator();
	sharesSyncAction->plug(sharesMenu);

	// create Settings menu
	KPopupMenu *settingsMenu = new KPopupMenu(this);
	settingsOptionsAction->plug(settingsMenu);

	// assign Help menu
	KPopupMenu *helpMenu_ = helpMenu();

	// insert menus into the menubar
	menuBar()->insertItem(i18n("&File"),     fileMenu,     0, ALT + Key_F);
	menuBar()->insertItem(i18n("&Shares"),   sharesMenu,   0, ALT + Key_S);
	menuBar()->insertItem(i18n("S&ettings"), settingsMenu, 0, ALT + Key_E);
	menuBar()->insertItem(i18n("&Help"),     helpMenu_,    0, ALT + Key_H);

	connect(m_settingsDialog, SIGNAL(sigOK()),              this,  SLOT(saveConfig()));
	connect(m_settingsDialog, SIGNAL(sigShutApollon(bool)), this,  SLOT(setShutdownApollon(bool)));
	connect(m_settingsDialog, SIGNAL(sigStartDocked(bool)), this,  SLOT(setStartDocked(bool)));
	connect(m_settingsDialog, SIGNAL(sigShowPopUps(bool)),  this,  SLOT(setShowPopUps(bool)));
	connect(m_settingsDialog, SIGNAL(sigStartPage(int)),    this,  SLOT(setStartPage(int)));
	connect(m_settingsDialog, SIGNAL(sigRestartDaemon()),   m_gcn, SLOT(sltRestartDaemon()));
	connect(m_settingsDialog, SIGNAL(sigKillDaemon(bool)),  this,  SLOT(setKillGiFT(bool)));
}

void ApollonApp::initCentre()
{
	KIconLoader *loader = KGlobal::iconLoader();

	m_tabWidget = new KJanusWidget(this, "tabw", KJanusWidget::IconList);
	setCentralWidget(m_tabWidget);

	QVBox *infoBox   = m_tabWidget->addVBoxPage(i18n("Info"),           NULL, loader->loadIcon("messagebox_info", KIcon::Small, 32) );
	QVBox *searchBox = m_tabWidget->addVBoxPage(i18n("Search"),         NULL, loader->loadIcon("find",            KIcon::Small, 32));
	QVBox *transBox  = m_tabWidget->addVBoxPage(i18n("Transfers"),      NULL, loader->loadIcon("network",         KIcon::Small, 32) );
	QVBox *sharedBox = m_tabWidget->addVBoxPage(i18n("Shared folders"), NULL, loader->loadIcon("folder",          KIcon::Small,32));
	QVBox *playerBox = m_tabWidget->addVBoxPage(i18n("Player"),         NULL, loader->loadIcon("kmix",            KIcon::Small, 32));

	m_infoTab       = new ApollonInfoTab(      infoBox,   "ApollonInfoTab",     m_gcn);
	m_searchTab     = new ApollonSearchTab(    searchBox, "ApollonSearchTab",   m_gcn);
	m_transTab      = new ApollonTransferTab(  transBox,  "ApollonTransferTab", m_gcn);
	m_sharedViewTab = new ApollonSharedViewTab(sharedBox, "ApollonSharedViewTab");
	m_playerTab     = new ApollonPlayerTab(    playerBox, "ApollonPlayerTab",   m_tabWidget, 4);

	m_transTab->setPreferences(m_settingsDialog);
	m_transTab->setPlayerTab(m_playerTab);
	m_sharedViewTab->setPlayerTab(m_playerTab);

	connect(m_settingsDialog, SIGNAL(sigAutoSearchInterval(int)), m_transTab,      SLOT(sltAutoSearchInterval(int)));
	connect(m_settingsDialog, SIGNAL(sigSharedFolder(const QString &)),  m_sharedViewTab, SLOT(addFolder(const QString &)));
	connect(m_infoTab,        SIGNAL(sigKill(bool)),              this,            SLOT(setKillGiFT(bool)));
	connect(m_infoTab,        SIGNAL(statusMessage(QString)),     this,            SLOT(setStatus(QString)));
	connect(m_transTab,       SIGNAL(browseUser(QString)),        m_searchTab,     SLOT(browseUser(QString)));
	connect(m_transTab,       SIGNAL(browseUser(QString)),        this,            SLOT(showSearches(QString)));
	connect(m_transTab,       SIGNAL(statsUpdate(int, float)),    this,            SLOT(statsUpdate(int, float)));
}

void ApollonApp::setStatus(QString text)
{
	statusBar()->clear();
	statusBar()->message(text);
}

void ApollonApp::sharesSync()
{
	m_gcn->makeShareRequest("sync");
}

void ApollonApp::sharesHide()
{
	m_gcn->makeShareRequest("hide");
}

void ApollonApp::sharesShow()
{
	m_gcn->makeShareRequest("show");
}

void ApollonApp::giftShareReply(QString reply)
{
	if(reply.find("status") > -1)
	{
		setStatus(i18n("Synchronizing shares: ") + giFTConnection::searchKey("status", reply));

		if(reply.find("Done") > -1)
		{
			m_gcn->makeStatsRequest();
		}
	}
	else
	{
		if(reply.find("hide") > -1)
		{
			setStatus(i18n("Shares are now hidden."));
			m_gcn->makeStatsRequest();
		}
		else
		{
			setStatus(i18n("Shares are now shown."));
			m_gcn->makeStatsRequest();
		}
	}
}

void ApollonApp::showSearches(QString)
{
	m_tabWidget->showPage(1);
}

void ApollonApp::initSysTray()
{
	connect(this, SIGNAL(sig_isWM_KDE(bool)), m_transTab, SLOT(slotCheckWM(bool)));
	
	bool kde_running = checkForWM();
	
	emit sig_isWM_KDE(kde_running);
	
	if (kde_running){
		m_systemTray = new ApollonSystemTray(this, "systemTray");
		m_systemTray->show();
		m_transTab->setTray(m_systemTray);

		connect(m_systemTray, SIGNAL(quitSelected()),                this,         SLOT(quit()));
		connect(m_transTab,   SIGNAL(statsUpdate(int, float)), m_systemTray, SLOT(statsUpdate(int, float)));
		}
	else {
		m_systemTrayGnome = new ApollonSystemTrayGnome(this, "systemTray");
		m_systemTrayGnome->show();
		m_transTab->setTray(m_systemTrayGnome);

		connect(m_systemTrayGnome, SIGNAL(quitSelected()),                this,         SLOT(quit()));
		connect(m_transTab,   SIGNAL(statsUpdate(int, float)), m_systemTrayGnome, SLOT(statsUpdate(int, float)));
	}
}

void ApollonApp::setGiFTpid(pid_t pid)
{
	m_giFTpid = pid;
}

void ApollonApp::setKillGiFT(bool killGiFT)
{
	m_killGiFT = killGiFT;
}

void ApollonApp::setShowPopUps(bool show)
{
	m_transTab->setShowPopup(show);
}

void ApollonApp::setStartPage(int page)
{
	m_tabWidget->showPage(page);
}

bool ApollonApp::checkForWM()
{
	KProcess *proc = new KProcess;
	*proc << "dcop";
	*proc << "kdesktop";
	proc->start(KProcess::Block);
	return !(proc->exitStatus());
}



