
/***************************************************************************
                          ApollonSharedViewTab.h  -  description
                             -------------------
begin		: June 05 2003
copyright	: (C) 2003 Martin Koebele
email		: apollon@mkoebele.de
description	: 

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <assert.h>
#include <cstdlib>

#include <dcopclient.h>
#include <kapplication.h>
#include <kdirselectdialog.h>
#include <kfiletreeview.h>
#include <kiconloader.h>
#include <kimagefilepreview.h>
#include <kglobal.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <kpopupmenu.h>
#include <kprocess.h>
#include <kurl.h>

#include <qcstring.h>
#include <qdir.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qlistview.h>
#include <qpixmap.h>
#include <qpushbutton.h>
#include <qregexp.h>
#include <qsizepolicy.h>
#include <qsplitter.h>
#include <qstring.h>

#include "apollonplayertab.h"
#include "apollonsharedviewtab.h"


ApollonSharedViewTab::ApollonSharedViewTab(QWidget *parent, const QString &/*name*/) :
  ApollonTab(parent),
  m_playerTab(0)
{
	KIconLoader *loader = KGlobal::iconLoader();

	QSplitter *splitter = new QSplitter(this);
	splitter->setOrientation(QSplitter::Horizontal);

	m_listView = new KFileTreeView(splitter);
	m_listView->setSelectionMode(QListView::Extended);
	m_listView->addColumn(i18n("Folders"));
	m_listView->setAllColumnsShowFocus(TRUE);
	m_listView->setRootIsDecorated(TRUE);
	m_listView->setMaximumWidth(350);
	m_listView->setShowSortIndicator(TRUE);
	m_listView->setItemsMovable(TRUE);
	m_listView->setSorting(0, TRUE);

	m_kpv = new KImageFilePreview(splitter);

	splitter->show();

	connect(m_listView, SIGNAL(doubleClicked(QListViewItem *,const QPoint &, int)),       this, SLOT(doubleClicked(QListViewItem *, const QPoint &, int) ));
	connect(m_listView, SIGNAL(rightButtonPressed(QListViewItem *, const QPoint &, int)), this, SLOT(popUp(QListViewItem *, const QPoint &, int)));
	connect(m_listView, SIGNAL(itemRenamed(QListViewItem *, int)),                        this, SLOT(rename(QListViewItem *, int)));
	connect(m_listView, SIGNAL(currentChanged(QListViewItem *)),                          this, SLOT(preview(QListViewItem *)));

	m_popupMenu = new KPopupMenu(m_listView);
	m_popupMenu->insertItem(loader->loadIcon("player_play", KIcon::Small), i18n("&Play"),                 this, SLOT(playFile()),      Key_Return, 0);
	m_popupMenu->insertSeparator();
	m_popupMenu->insertItem(loader->loadIcon("xmms",        KIcon::Small), i18n("Add to &XMMS playlist"), this, SLOT(addFileToXMMS()), Key_X,      1);
	m_popupMenu->insertItem(loader->loadIcon("juk",         KIcon::Small), i18n("Add to &JuK playlist"),  this, SLOT(addFileToJuK()),  Key_J,      2);
	m_popupMenu->insertSeparator();
	m_popupMenu->insertItem(loader->loadIcon("editdelete",  KIcon::Small), i18n("&Delete file"),          this, SLOT(deleteFile()),    Key_Delete, 3);
	m_popupMenu->insertItem(                                               i18n("&Rename file"),          this, SLOT(renameFile()),    Key_F2,     4);
	m_popupMenu->setItemEnabled(4, true);

	m_id = 0;
	m_moveToMenu = new KPopupMenu(m_listView);
	m_moveToMenu->insertItem(i18n("Browse..."), m_id++);
	m_moveToMenu->insertSeparator();
	m_popupMenu->insertItem(i18n("Move to: ..."), m_moveToMenu);

	connect(m_moveToMenu, SIGNAL(activated(int)), this, SLOT(moveTo(int)));
}

ApollonSharedViewTab::~ApollonSharedViewTab()
{
}

void ApollonSharedViewTab::addFolder(const QString &folder)
{
	QString path = folder;
	if(path != "none")
	{
		QString home(getenv("HOME"));
		path = path.replace(QString("~"), home);

		KFileTreeBranch *branch = m_listView->addBranch(KURL(path), path);
		branch->root();
		KIconLoader *loader = KGlobal::iconLoader();
		m_moveToMenu->insertItem(loader->loadIcon("folder", KIcon::Small), path, m_id++);
  	}
}

void ApollonSharedViewTab::doubleClicked(QListViewItem *item, const QPoint &/*point*/, int /*col*/)
{
	play((KFileTreeViewItem *) item);
}

void ApollonSharedViewTab::popUp(QListViewItem */*item*/, const QPoint &point, int /*col*/)
{
	m_popupMenu->popup(point);
}

void ApollonSharedViewTab::playFile()
{
	m_currentItem = (KFileTreeViewItem *) m_listView->currentItem();

	if(m_currentItem->isDir())
		KMessageBox::sorry(this, i18n("Sorry, I can't play directories."), i18n("Sorry"));
	else
		play(m_currentItem);
}

void ApollonSharedViewTab::addFileToXMMS()
{
	m_currentItem = (KFileTreeViewItem *) m_listView->firstChild();
	m_currentItem = (KFileTreeViewItem *) m_currentItem->firstChild();
	while(m_currentItem != 0)
	{
		if(m_currentItem->isSelected())
		{
			KProcess xmmsCommand;
			xmmsCommand << "xmms" << "--enqueue" << m_currentItem->url().path();
			xmmsCommand.start(KProcess::DontCare);
		}
		m_currentItem = (KFileTreeViewItem *) m_currentItem->nextSibling();
	}
}

void ApollonSharedViewTab::addFileToJuK()
{
	m_currentItem = (KFileTreeViewItem *) m_listView->firstChild();
	m_currentItem = (KFileTreeViewItem *) m_currentItem->firstChild();
	while (m_currentItem != 0)
	{
		if (m_currentItem->isSelected())
		{
			QString file_url = m_currentItem->url().path();

			if(!kapp->dcopClient()->send("juk", "Player", "openFile(QString)", file_url))
			{
				KMessageBox::sorry(this, i18n("I couldn't contact JuK."), i18n("Sorry"));
			}
		}
		m_currentItem = (KFileTreeViewItem *) m_currentItem->nextSibling();
	}
}

void ApollonSharedViewTab::play(KFileTreeViewItem *item)
{
	assert(m_playerTab != 0);

	m_playerTab->play(item->url());
}

void ApollonSharedViewTab::deleteFile()
{
	QDir dir;
	m_currentItem = (KFileTreeViewItem *) m_listView->firstChild();
	m_currentItem = (KFileTreeViewItem *) m_currentItem->firstChild();
	while(m_currentItem != 0)
	{
		if(m_currentItem->isSelected())
		{
			if(!m_currentItem->isDir())
			{
				int answer = KMessageBox::warningYesNo(this, i18n("Do you want to delete\n?" + m_currentItem->path()), i18n("Deletion"));
				if(answer == KMessageBox::Yes)
				{
					dir.remove(m_currentItem->path());
				}
			}
			else
			{
				KMessageBox::sorry(this, i18n("Deleting an entire directory?\n (has no effect)"), i18n("Sorry"));
			}
		}
		m_currentItem = (KFileTreeViewItem *) m_currentItem->nextSibling();
	}
}

void ApollonSharedViewTab::renameFile()
{
	m_popupMenu->setAccel(0, 0);
	m_currentItem = m_listView->currentKFileTreeViewItem();
	m_currentItem->setRenameEnabled(0, true);

	if(m_currentItem->renameEnabled(0))
	{
		m_currentItem->startRename(0);
	}
	else
	{
		KMessageBox::sorry(this, i18n("Couldn't rename file or directory"), i18n("Sorry"));
	}
}

void ApollonSharedViewTab::rename(QListViewItem *item, int col)
{
	QString newName = item->text(col);
	KURL url = m_currentItem->path();
	QString newFileName = url.directory() + QString("/") + newName;
	QDir dir;
	if(!dir.rename(m_currentItem->path(), newFileName))
	{
		KMessageBox::error(this, i18n("Couldn't rename file or directory!"), i18n("Sorry"));
	}
	m_popupMenu->setAccel(Key_Return, 0);
}

void ApollonSharedViewTab::moveTo(int id)
{
	m_currentItem = m_listView->currentKFileTreeViewItem();

	if(id == 0) // browse
	{
		KURL newUrl = KDirSelectDialog::selectDirectory();

		if(!newUrl.isEmpty())
		{
			KURL url = m_currentItem->path();
			QString filename = url.fileName();
			QDir dir;
			QString newPath = newUrl.path() + QString("/") + filename;
			QString oldPath = url.path();
			if(!dir.rename(oldPath, newPath))
			{
				KMessageBox::error(this, i18n("Couldn't move file or directory!"), i18n("Error"));
			}
		}
	}
	else
	{
		m_currentItem = m_listView->currentKFileTreeViewItem();
		KURL url = m_currentItem->path();
		QString filename = url.fileName();
		QDir dir;
		QString newPath = m_moveToMenu->text(id) + QString("/") + filename;
		QString oldPath = url.path();
		if(!dir.rename(oldPath, newPath))
		{
			KMessageBox::error(this, i18n("Couldn't move file or directory!"), i18n("Error"));
		}
	}
}

void ApollonSharedViewTab::preview(QListViewItem *item)
{
	m_currentItem = (KFileTreeViewItem *) item;
	KURL url = m_currentItem->path();
	m_kpv->showPreview(url);
}
