/***************************************************************************
                          apollonsystemtray.cpp  -  description
                             -------------------
    copyright            : (C) 2003 Martin Koebele
    email                : martin@mkoebele.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kaction.h>
#include <kapplication.h>
#include <kiconloader.h>
#include <kmainwindow.h>
#include <kpopupmenu.h>
#include <klocale.h>
#include <kwin.h>
#include <kwinmodule.h>

#include <qstring.h>

#include "apollonsystemtraygnome.h"

ApollonSystemTrayGnome::ApollonSystemTrayGnome(QWidget *parent, const char *name) :
  TrayIcon(parent, name),
  m_parent((KMainWindow *) parent),
  m_counter(0),
  m_activeDownloads(0),
  m_rate(0)
{
	m_icon = SmallIcon("apollon");
	setIcon(m_icon);

	KAction *minimizeAction = new KAction(i18n("&Minimize"), 0, this, SLOT(minimizeRestoreAction()), m_parent->actionCollection(), "minimizeRestoreAction");
	KAction *quitAction =     KStdAction::quit(                 this, SIGNAL(closed()),              m_parent->actionCollection(), "quitAction");

	m_popupMenu = new KPopupMenu();
	m_popupMenu->insertTitle(SmallIcon("apollon"), i18n("Apollon"));
	minimizeAction->plug(m_popupMenu);
	quitAction->plug(m_popupMenu);

	setPopup(m_popupMenu);

	KWin::Info info = KWin::info(m_parent->winId());
	m_onAllDesktops = info.onAllDesktops;

	statsUpdate(0, 0.0);

	connect(this, SIGNAL(clicked(const QPoint &, int)), this, SLOT(activateOrHide(const QPoint &, int)));
}

ApollonSystemTrayGnome::~ApollonSystemTrayGnome()
{
	delete m_popupMenu;
}

void ApollonSystemTrayGnome::statsUpdate(int activeDownloads, float rate)
{
	m_activeDownloads = activeDownloads;
	m_rate = rate;
	m_statusText = i18n("1 active download at %1 KB/s",
	                    "%n active downloads at %1 KB/s", m_activeDownloads).arg(m_rate);

	setToolTip(m_statusText);
}

void ApollonSystemTrayGnome::activateOrHide(const QPoint &, int)
{
	KWin::Info info = KWin::info(m_parent->winId());

	bool mapped = (info.mappingState != NET::Withdrawn);
	//    - not mapped -> show, raise, focus
	//    - mapped
	//        - obscured -> raise, focus
	//        - not obscured -> hide
	if(!mapped)
	{
		minimizeRestore(true);
	}
	else
	{
		if(info.desktop != KWin::currentDesktop())
		{
			KWin::setCurrentDesktop(info.desktop);
			KWin::setActiveWindow(m_parent->winId());
			return;
		}

		KWinModule module;
		for(QValueList<WId>::ConstIterator it = module.stackingOrder().fromLast();
		    it != module.stackingOrder().end() && (*it) != m_parent->winId();
		    --it)
		{
			KWin::Info info = KWin::info(*it/*, NET::WMGeometry | NET::XAWMState*/);
			if(info.mappingState == NET::Visible && info.geometry.intersects(m_parent->geometry()))
			{
				m_parent->raise();
				KWin::setActiveWindow(m_parent->winId());
				return;
			}
		}
		minimizeRestore(false);
	}
}

void ApollonSystemTrayGnome::minimizeRestoreAction()
{
	bool restore = !(m_parent->isVisible());
	minimizeRestore(restore);
}

void ApollonSystemTrayGnome::minimizeRestore(bool restore)
{
	KWin::Info info = KWin::info(m_parent->winId()/*, NET::WMGeometry | NET::WMDesktop*/);
	KAction *action = m_parent->actionCollection()->action("minimizeRestoreAction");

	if(restore)
	{
		if(m_onAllDesktops)
			KWin::setOnAllDesktops(m_parent->winId(), true);
		else
			KWin::setOnDesktop(m_parent->winId(), KWin::currentDesktop());
		m_parent->move(info.geometry.topLeft()); // avoid placement policies
		m_parent->show();
		m_parent->raise();
		KWin::setActiveWindow(m_parent->winId());

		action->setText(i18n("&Minimize"));
	}
	else
	{
		m_onAllDesktops = info.onAllDesktops;
		m_parent->hide();

		action->setText(i18n("&Restore"));
	}
}
