/***************************************************************************
                          Apollontransfertab.cpp  -  description
                             -------------------
    begin                : Tue Apr 2 2002
    copyright            : (C) 2002 by Robert Wittams
    email                : robert@wittams.com en Je moeder
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <assert.h>

#include <kconfig.h>
#include <kdebug.h>
#include <kglobal.h>
#include <kiconloader.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <kpassivepopup.h>

#include <qdir.h>
#include <qlabel.h>
#include <qpopupmenu.h>
#include <qsplitter.h>
#include <qtimer.h>
#include <qtooltip.h>
#include <qvbox.h>

#include "giftconnection.h"
#include "apollonplayertab.h"
#include "apollonsystemtray.h"
#include "apollonsystemtraygnome.h"
#include "apollontransfertab.h"
#include "apollontransferviewitem.h"
#include "apollonutils.h"
#include "preferences.h"
#include "qpassivepopup.h"

//#include "kpp.h"


ApollonTransferTab::ApollonTransferTab(QWidget *parent, const char *name, giFTConnection *gcn) :
  ApollonTab(parent, name, gcn),
  m_playerTab(0)
{
	KIconLoader *loader = KGlobal::iconLoader();

	connect(m_gcn, SIGNAL(giftAddDownloadReply(QString)), this, SLOT(addDownloadViewItem(QString)));
	connect(m_gcn, SIGNAL(giftAddUploadReply(QString)),   this, SLOT(addUploadViewItem(QString)));
	connect(m_gcn, SIGNAL(giftChgUploadReply(QString)),   this, SLOT(updateUploadView(QString)));
	connect(m_gcn, SIGNAL(giftChgDownloadReply(QString)), this, SLOT(updateDownloadView(QString)));
	connect(m_gcn, SIGNAL(giftDelDownloadReply(QString)), this, SLOT(delDownloadViewItem(QString)));
	connect(m_gcn, SIGNAL(giftDelUploadReply(QString)),   this, SLOT(delUploadViewItem(QString)));
	connect(m_gcn, SIGNAL(giftAddSourceReply(QString)),   this, SLOT(addSourceItem(QString)));
	connect(m_gcn, SIGNAL(giftDelSourceReply(QString)),   this, SLOT(delSourceItem(QString)));
	connect(m_gcn, SIGNAL(giftItemReply(QString)),        this, SLOT(itemReply(QString)));
	connect(m_gcn, SIGNAL(disconnected()),                this, SLOT(disconnected()));

	QSplitter *splitter = new QSplitter(this);
	splitter->setOrientation(QSplitter::Vertical);

	//setMinimumWidth(400);
	QVBox *downBox = new QVBox(splitter);

	new QLabel(i18n("Downloads"), downBox);
	m_downList = new ApollonListView(downBox);

	m_downList->setSelectionMode(QListView::Extended);

	m_downList->addColumn(i18n("Name"));
	m_downList->setColumnWidthMode(0, QListView::Manual);
	m_downList->setColumnWidth(0, 200);

	m_downList->addColumn(i18n("Size"));
	m_downList->setColumnAlignment(1, Qt::AlignRight);

	m_downList->addColumn(i18n("Transferred"));
	m_downList->setColumnAlignment(2, Qt::AlignRight);

	m_downList->addColumn(i18n("Bandwidth"));
	m_downList->setColumnAlignment(3, Qt::AlignRight);

	m_downList->addColumn(i18n("Progress"));
	m_downList->setColumnAlignment(4, Qt::AlignRight);

	m_downList->addColumn(i18n("ETA"));
	m_downList->setColumnAlignment(5, Qt::AlignRight);

	m_downList->setAllColumnsShowFocus(true);
	m_downList->setRootIsDecorated(true);
	m_downList->setShowSortIndicator(true);

	m_downMenu = new QPopupMenu(m_downList);
	m_downMenu->insertItem(loader->loadIcon("player_play",   KIcon::Small), i18n("&Preview"),                 this, SLOT(previewFile()),    Key_Return, 0, 0);
	m_downMenu->insertItem(loader->loadIcon("connect_no",    KIcon::Small), i18n("P&ause"),                   this, SLOT(pauseDownload()),  Key_P, 1, 1);
	m_downMenu->insertItem(loader->loadIcon("viewmag+",      KIcon::Small), i18n("&Find more sources"),       this, SLOT(sourceSearch()),   Key_Plus, 2, 2);
	m_downMenu->insertItem(loader->loadIcon("stop",          KIcon::Small), i18n("&Cancel Download"),         this, SLOT(cancelDownload()), Key_Delete, 3, 3);
	m_downMenu->insertSeparator();
	m_downMenu->insertItem(loader->loadIcon("trashcan_full", KIcon::Small), i18n("C&lear Finished/Canceled"), this, SLOT(clearFinishedDownloads()));

	m_browseUserMenu = new QPopupMenu(m_downList);

	connect(m_downList, SIGNAL(rightButtonPressed(QListViewItem*, const QPoint&, int)), this, SLOT(downloadPopup(QListViewItem*, const QPoint&, int)));
	connect(m_downList, SIGNAL(expanded(QListViewItem *)), this, SLOT(updateDownloadSources(QListViewItem *)));

	QVBox *upBox = new QVBox(splitter);

	new QLabel(i18n("Uploads"), upBox);
	m_upList = new ApollonListView(upBox);
	m_upList->addColumn(i18n("Name"));
	m_upList->setColumnWidthMode(0, QListView::Manual);
	m_upList->setColumnWidth(0, 200);
	m_upList->addColumn(i18n("Size"));
	m_upList->setColumnAlignment(1, Qt::AlignRight);

	m_upList->addColumn(i18n("Transferred"));
	m_upList->setColumnAlignment(2, Qt::AlignRight);

	m_upList->addColumn(i18n("Bandwidth"));
	m_upList->setColumnAlignment(3, Qt::AlignRight);

	m_upList->addColumn(i18n("Progress"));
	m_upList->setColumnAlignment(4, Qt::AlignRight);

	m_upList->addColumn(i18n("ETA"));
	m_upList->setColumnAlignment(5, Qt::AlignRight);

	m_upList->setAllColumnsShowFocus(true);
	m_upList->setRootIsDecorated(true);
	m_upList->setShowSortIndicator(true);

	m_upMenu = new QPopupMenu(m_upList);
	m_upMenu->insertItem(loader->loadIcon("stop",          KIcon::Small), i18n("&Cancel Upload"),           this, SLOT(cancelUpload()));
	m_upMenu->insertSeparator();
	m_upMenu->insertItem(loader->loadIcon("trashcan_full", KIcon::Small), i18n("C&lear Finished/Canceled"), this, SLOT(clearFinishedUploads()));

	connect(m_upList, SIGNAL(rightButtonPressed(QListViewItem *, const QPoint &, int)), this, SLOT(uploadPopup(QListViewItem *, const QPoint &, int)));
	connect(m_upList, SIGNAL(expanded(QListViewItem *)), this, SLOT(updateUploadSources(QListViewItem *)));

	splitter->setResizeMode(upBox, QSplitter::KeepSize);
	splitter->show();
	m_showPopup = true;

	KConfig *config = KGlobal::config();
	m_downList->restoreLayout(config, "TransferTabDown");
	m_upList->restoreLayout(config, "TransferTabUp");

}


ApollonTransferTab::~ApollonTransferTab()
{
	KConfig *config = KGlobal::config();
	m_downList->saveLayout(config, "TransferTabDown");
	m_upList->saveLayout(config, "TransferTabUp");
	config->sync();
}


void ApollonTransferTab::addDownloadViewItem(QString line)
{
	int     ID =       giFTConnection::searchKey("ADDDOWNLOAD", line).toInt();
	int     transmit = giFTConnection::searchKey("transmit", line).toInt();
	int     size =     giFTConnection::searchKey("size", line).toInt();
	QString hash =     giFTConnection::searchKey("hash", line);
	QString file =     giFTConnection::searchKey("file", line);
	QString protocol_name = protocolName(line);

	float   progress = 100 * ((float) transmit / (float) size);

	ApollonTransferViewItem *item = new ApollonTransferViewItem(m_downList, "download", ID, protocol_name);

	item->setHash(hash);
	item->setName(file);
	item->setSize(size);
	item->setTransmit(transmit);
	//item->setStatus(QString("%1%").arg(progress, 0, 'f', 1));
	item->setProgress(progress);
	item->setBandwidth("--");

	int numberofsources = line.contains("SOURCE {", true);

	for(int i = 0; i < numberofsources; i++)
	{
		QString source = line.section("SOURCE {", i + 1, i + 1);
		ApollonTransferViewItem *sourceitem = new ApollonTransferViewItem(item);

		transmit = giFTConnection::searchKey("transmit", source).toInt();
		progress = 100 * ((float) transmit / (float) size);

		sourceitem->setName(giFTConnection::searchKey("user", source));

		sourceitem->setTransmit(transmit);
		sourceitem->setStart(giFTConnection::searchKey("start", source).toInt());
		sourceitem->setStatus(giFTConnection::searchKey("status", source));
		sourceitem->setProgress(progress);
		sourceitem->setUrl(giFTConnection::searchKey("url", source));
		sourceitem->setID(ID);
	}
}

void ApollonTransferTab::delDownloadViewItem(QString line)
{
	int ID = giFTConnection::searchKey("DELDOWNLOAD", line).toInt();
	ApollonTransferViewItem *item = ApollonTransferViewItem::getItem(m_downList, ID);
	if(item == 0)
	{
		kdDebug() << "Received an invalid delDownloadView request." << endl;
		return;
	}

	if(item->size() == item->transmit())
	{
		item->setStatus(i18n("Completed"));
		item->setProgress(-1);

		if(m_showPopup)
		{
			KIconLoader *loader = KGlobal::iconLoader();
			QMimeSourceFactory::defaultFactory()->setPixmap("Apollon",loader->loadIcon("apollon", KIcon::MainToolbar));
			QString statustext;
			statustext = "<br><img source = \"Apollon\"><br><strong>" + i18n("Download finished:") + "</strong> <br>" + item->name() +"<br>";
			if (m_isWM_KDE)
				QPassivePopup::message(statustext, m_systemTray);
			else
				QPassivePopup::message(statustext, m_systemTrayGnome->widget());
		}
	}
	else
	{
		item->setStatus(i18n("Canceled"));
		item->setProgress(-1);
	}
}

void ApollonTransferTab::addUploadViewItem(QString line)
{
	int     ID =       giFTConnection::searchKey("ADDUPLOAD", line).toInt();
	int     transmit = giFTConnection::searchKey("transmit", line).toInt();
	int     size =     giFTConnection::searchKey("size", line).toInt();
	QString file =     giFTConnection::searchKey("file", line);
	QString protocol_name = protocolName(line);
	float   progress = 100 * ((float) transmit / (float) size);

	ApollonTransferViewItem *item = new ApollonTransferViewItem(m_upList, "upload", ID, protocol_name);

	item->setName(file);
	item->setSize(size);
	item->setTransmit(transmit);
	//item->setStatus(QString("%1 %").arg(progress, 0, 'f', 1) );
	item->setProgress(progress);
	int numberofsources = line.contains("SOURCE {", true);

	for(int i = 0; i < numberofsources; i++)
	{
		QString source = line.section("SOURCE {", i + 1, i + 1);

		transmit = giFTConnection::searchKey("transmit", source).toInt();
		progress = 100 * ((float) transmit / (float) size);

		ApollonTransferViewItem *sourceitem = new ApollonTransferViewItem(item);

		sourceitem->setName(giFTConnection::searchKey("user", source));
		sourceitem->setSize(transmit);
		sourceitem->setStatus(giFTConnection::searchKey("status", source));
		sourceitem->setProgress(progress);
	}
}

void ApollonTransferTab::delUploadViewItem(QString line)
{
	int ID = giFTConnection::searchKey("DELUPLOAD", line).toInt();

	ApollonTransferViewItem *item = ApollonTransferViewItem::getItem(m_upList, ID);
	if(item == 0)
	{
		kdDebug() << "Received an invalid delUploadViewItem request." << endl;
		return;
	}

	if(item->size() == item->transmit())
	{
		item->setStatus(i18n("Completed"));
		item->setProgress(-1);
	}
	else
	{
		item->setStatus(i18n("Canceled"));
		item->setProgress(-1);
	}
}

void ApollonTransferTab::updateDownloadView(QString line)
{
	int     ID =        giFTConnection::searchKey("CHGDOWNLOAD", line).toInt();
	int     transmit =  giFTConnection::searchKey("transmit", line).toInt();
	int     through =   giFTConnection::searchKey("throughput", line).toInt();
	int     elapsed =   giFTConnection::searchKey("elapsed", line).toInt();
	int     size =      giFTConnection::searchKey("size", line).toInt();
	if(transmit == through) through = 0;
	QString bandwidth = ApollonUtils::calcBandwidth(through, elapsed);
	QString protocol_name = protocolName(line);

	float   progress =  100 * ((float) transmit / (float) size);

	ApollonTransferViewItem *item = ApollonTransferViewItem::getItem(m_downList, ID);
	if(item == 0)
	{
		kdDebug() << "Received an invalid updateDownloadView request." << endl;
		return;
	}
	item->updateIcon(protocol_name, "download");
	item->setTransmit(transmit);
	item->setBandwidth(bandwidth);
	item->setProgress(progress);
	item->setETA(through == 0 ? -1 : (int) ((size - transmit) / 1000 * ((float) elapsed / through)));

	item->setUpdateString(line);

	if(item->isOpen())
	{
		updateDownloadSources(item);
	}

	countActives();
}

void ApollonTransferTab::updateDownloadSources(QListViewItem *item)
{
	QString line = dynamic_cast<ApollonTransferViewItem *>(item)->updateString();

	int     elapsed =   giFTConnection::searchKey("elapsed", line).toInt();

	QString source = line.section("SOURCE {", 0, 0);
	line.remove(0, source.length());

	ApollonTransferViewItem *sourceItem = dynamic_cast<ApollonTransferViewItem *>(item)->firstChild();

	while(source = line.section("SOURCE {", 1, 1), !source.isEmpty())
	{
		if(sourceItem == 0)
		{
			sourceItem = new ApollonTransferViewItem(item);
		}

		QString sourceName =   giFTConnection::searchKey("user", source);
		QString sourceStatus = giFTConnection::searchKey("status", source);

		sourceItem->setName(sourceName);
		sourceItem->setStatus(sourceStatus);
		sourceItem->setStart(giFTConnection::searchKey("start", source).toInt());
		if(sourceStatus == "Active")
		{
			int sourceTransmit = giFTConnection::searchKey("transmit", source).toInt();
			int sourceSize = giFTConnection::searchKey("total", source).toInt();
			int aux = sourceTransmit - sourceItem->transmit();

			sourceItem->setTransmit(sourceTransmit);
			sourceItem->setBandwidth(ApollonUtils::calcBandwidth(aux, elapsed));
			sourceItem->setProgress(100 * ((float) sourceTransmit / (float) sourceSize));
		}
		else
		{
			sourceItem->setTransmit(-1);
			sourceItem->setBandwidth("--");
			sourceItem->setProgress(-1);
		}

		sourceItem = sourceItem->nextSibling();

		line.remove(0, source.length() + 8);
	}
}

void ApollonTransferTab::updateUploadView(QString line)
{
	int     ID =       giFTConnection::searchKey("CHGUPLOAD", line).toInt();
	int     transmit = giFTConnection::searchKey("transmit", line).toInt();
	int     through =  giFTConnection::searchKey("throughput", line).toInt();
	int     elapsed =  giFTConnection::searchKey("elapsed", line).toInt();
	int     size =     giFTConnection::searchKey("size", line).toInt();
	float   progress = 100 * ((float) transmit / (float) size);

	ApollonTransferViewItem *item = ApollonTransferViewItem::getItem(m_upList, ID);
	if(item == 0)
	{
		kdDebug() << "Received an invalid updateUploadView request." << endl;
		return;
	}

	item->setTransmit(transmit);
	item->setBandwidth(ApollonUtils::calcBandwidth(through, elapsed));
	//item->setStatus(QString("%1 %").arg(progress, 0, 'f', 1));
	item->setProgress(progress);
	item->setETA(through == 0 ? -1 : ((int) ((size - transmit) / 1000 * ((float) elapsed / through))));

	item->setUpdateString(line);

	updateUploadSources(item);
}

void ApollonTransferTab::updateUploadSources(QListViewItem *item)
{
	QString line = dynamic_cast<ApollonTransferViewItem *>(item)->updateString();

	QString source = line.section("SOURCE {", 0, 0);
	line.remove(0, source.length());

	ApollonTransferViewItem *sourceItem = dynamic_cast<ApollonTransferViewItem *>(item)->firstChild();

	while(source = line.section("SOURCE {", 1, 1), !source.isEmpty())
	{
   		if(sourceItem == 0)
		{
			sourceItem = new ApollonTransferViewItem(item);
		}

		QString sourceName =   giFTConnection::searchKey("user", source);
		QString sourceStatus = giFTConnection::searchKey("status", source);

		sourceItem->setName(sourceName);
		sourceItem->setStatus(sourceStatus);

		if(sourceStatus == i18n("Active"))
		{
			int sourceTransmit = giFTConnection::searchKey("transmit", source).toInt();
			int sourceSize = giFTConnection::searchKey("total", source).toInt();

			sourceItem->setTransmit(sourceTransmit);
			sourceItem->setProgress(100 * ((float) sourceTransmit / (float) sourceSize));
		}
		else
		{
			sourceItem->setTransmit(-1);
			sourceItem->setProgress(-1);
		}

		sourceItem = sourceItem->nextSibling();

		line.remove(0, source.length() + 8);
	}
}

void ApollonTransferTab::addSourceItem(QString line)
{
	int ID = giFTConnection::searchKey("ADDSOURCE", line).toInt();

	ApollonTransferViewItem *item = ApollonTransferViewItem::getItem(m_downList, ID);
	QString protocol_name = protocolName(line);
	item->updateIcon(protocol_name, "download");

	if(item == 0)
	{
		kdDebug() << "Received an invalid addSourceItem request." << endl;
		return;
	}

	ApollonTransferViewItem *sourceItem = new ApollonTransferViewItem(item);

	sourceItem->setName(giFTConnection::searchKey("user", line));
	sourceItem->setStatus(giFTConnection::searchKey("status", line));
	sourceItem->setProgress(-1);
	sourceItem->setUrl(giFTConnection::searchKey("url", line));
	sourceItem->setID(ID);
}


void ApollonTransferTab::delSourceItem(QString line)
{
        int ID = giFTConnection::searchKey("DELSOURCE", line).toInt();

	ApollonTransferViewItem *item = ApollonTransferViewItem::getItem(m_downList, ID);
	if(item == 0)
	{
		kdDebug() << "Received an invalid addSourceItem request." << endl;
		return;
	}

	bool deleted = false;
	QString url = giFTConnection::searchKey("url", line);
	ApollonTransferViewItem *sourceItem = static_cast<ApollonTransferViewItem*>(item->firstChild());
	while (!deleted) {

	  if (sourceItem)
	    if (sourceItem->url() == url) {
	      item->takeItem(sourceItem);
	      delete sourceItem;
	      deleted = true;
	    }
	    else
	      sourceItem = sourceItem->nextSibling();
	  else
	    deleted = true;
	}
	    
}

void ApollonTransferTab::disconnected()
{
	clearAll();
}

void ApollonTransferTab::downloadPopup(QListViewItem *item, const QPoint &point, int)
{
	m_current = 0;

	if(!item)
	{
		return;
	}

	KIconLoader *loader = KGlobal::iconLoader();

	ApollonTransferViewItem* transview = (ApollonTransferViewItem*) item;

	m_browseUserMenu->clear();

	if (transview->parent() == 0)
	{
		m_current = transview->id();
		
		if (transview->isPaused())
		{
			m_downMenu->removeItem(1);
			m_downMenu->insertItem(loader->loadIcon("connect_established",   KIcon::Small), i18n("&Resume"),                 this, SLOT(unpauseDownload()),    Key_R, 1,1);
		}
		else
		{
			m_downMenu->removeItem(1);
			m_downMenu->insertItem(loader->loadIcon("connect_no",   KIcon::Small), i18n("P&ause"),                 this, SLOT(pauseDownload()),    Key_P, 1,1);
		}
		
		m_downMenu->popup(point);
	}
	else
	{
		m_current = transview->parent()->id();
		m_userIP =  transview->name();
		m_browseUserMenu->insertItem(loader->loadIcon("find", KIcon::Small), i18n("Browse user: ") + m_userIP, this, SLOT(sltBrowseUser()));
		m_browseUserMenu->insertItem(loader->loadIcon("stop", KIcon::Small), i18n("Remove this source user: ") + m_userIP, this, SLOT(removeSourceitem()));
		m_browseUserMenu->popup(point);
	}
}

void ApollonTransferTab::uploadPopup(QListViewItem *item, const QPoint &point, int)
{
	m_current = 0;

	if(!item)
	{
		return;
	}

	ApollonTransferViewItem *transview = (ApollonTransferViewItem*) item;

	if(transview->parent() == 0)
	{
		m_current = transview->id();
	}
	else
	{
		m_current = transview->parent()->id();
	}

	m_upMenu->popup(point);
}


void ApollonTransferTab::cancelDownload()
{
	QStringList strlist;
	ApollonTransferViewItem* item = (ApollonTransferViewItem *) m_downList->firstChild();
	while(item != 0)
	{
		if(item->isSelected())
		  {
			if(item->parent() == 0)
			{
				m_current = item->id();
			}
			else
			{
				m_current = item->parent()->id();
				item = item->parent();
			}
			strlist += item->text(0);
		}
		item = item->nextSibling();
	}

	int answer = KMessageBox::questionYesNoList (this, i18n("Do you want to delete these items?"),  strlist, i18n("Deletion"));
	if (answer == KMessageBox::Yes)
	{
		 item = (ApollonTransferViewItem *) m_downList->firstChild();
		while(item != 0)
		{
			if(item->isSelected())
			{
				if(item->parent() == 0)
				{
					m_current = item->id();
				}
				else
				{
					m_current = item->parent()->id();
					item = item->parent();
				}
 				m_gcn->makeTransferRequest(m_current, "cancel");
			}
			item = item->nextSibling();
		}
	}
}

void ApollonTransferTab::removeSourceitem()
{
	QString del_url;
	int finish=0;
	QStringList strlist;
	int del_id;
	ApollonTransferViewItem* item = (ApollonTransferViewItem *) m_downList->firstChild();
	while(item != 0)
		{
			while(!item->firstChild()){
			 if ( !item->nextSibling()){finish=1; break;

			 }else {item=item->nextSibling();}
			 }
			if (finish==1)break;
			item=item->firstChild();
			while(item != 0)
			{
				if(item->isSelected())
				{
					strlist += item->text(0);
				}
				if(item->nextSibling()) item=item->nextSibling();
				else break;
			}
			item=item->parent();
			item=item->nextSibling();
		}
	int answer = KMessageBox::questionYesNoList (this, i18n("Do you want to remove these sourcess?"),  strlist, i18n("Deletion"));
	if (answer == KMessageBox::Yes)
	{
		 item = (ApollonTransferViewItem *) m_downList->firstChild();
		 while(item != 0)
		{
			while(!item->firstChild()){
			 if ( !item->nextSibling()){finish=1; break;

			 }else {item=item->nextSibling();}
			 }
			if (finish==1)break;
			item=item->firstChild();
			while(item != 0)
			{
				if(item->isSelected())
				{
					del_id = item->id();
					del_url = item->url();
					m_gcn->makeDelSourceRequest(del_id,del_url);
				}
				if(item->nextSibling()) item=item->nextSibling();
				else break;
			}
			item=item->parent();
			item=item->nextSibling();
		}
	}
}

void ApollonTransferTab::pauseDownload()
{
	ApollonTransferViewItem* item = (ApollonTransferViewItem *) m_downList->firstChild();
	while(item != 0)
	{
		if(item->isSelected())
		{
			if(item->parent() == 0)
			{
				m_current = item->id();
				item->pauseItem();
			}
			else
			{
				m_current = item->parent()->id();
				item = item->parent();
			}
			m_gcn->makeTransferRequest(m_current, "pause");
		}
		item = item->nextSibling();
	}
}

void ApollonTransferTab::unpauseDownload()
{
	ApollonTransferViewItem* item = (ApollonTransferViewItem *) m_downList->firstChild();
	while(item != 0)
	{
		if(item->isSelected())
		{
			if(item->parent() == 0)
			{
				m_current = item->id();
				item->unpauseItem();
			}
			else
			{
				m_current = item->parent()->id();
				item = item->parent();
			}
			m_gcn->makeTransferRequest(m_current, "unpause");
		}
		item = item->nextSibling();
	}
}

void ApollonTransferTab::cancelUpload()
{
	m_gcn->makeTransferRequest(m_current, "cancel");
}

void ApollonTransferTab::clearFinishedDownloads()
{
	ApollonTransferViewItem *item = (ApollonTransferViewItem *) m_downList->firstChild();

	clearFinished(item);
}

void ApollonTransferTab::clearFinishedUploads()
{
	ApollonTransferViewItem *item = (ApollonTransferViewItem *) m_upList->firstChild();

	clearFinished(item);
}

void ApollonTransferTab::clearFinished(ApollonTransferViewItem *item)
{
	ApollonTransferViewItem *temp;

	while(item)
	{
		temp = item->nextSibling();
		if(item->status() == i18n("Canceled") || item->status() == i18n("Completed"))
		{
			delete item;
		}
		item = temp;
	}
}

void ApollonTransferTab::clearAll()
{
	ApollonTransferViewItem *p = (ApollonTransferViewItem *) m_downList->firstChild(), *q;
	while(p)
	{
		q = p->nextSibling();
		delete p;
		p = q;
	}

	p = (ApollonTransferViewItem *) m_upList->firstChild();
	while(p)
	{
		q = p->nextSibling();
		delete p;
		p = q;
	}
}

void ApollonTransferTab::previewFile()
{
	assert(m_playerTab != 0);
	assert(m_preferences != 0);

	QString url;

	if(((ApollonTransferViewItem *) m_downList->currentItem())->status() == i18n("Completed"))
	{
		url = QString("file:") + m_preferences->getCompletedPath() + "/" + ((ApollonTransferViewItem *) m_downList->currentItem())->name();
		m_playerTab->play(url);
	}
	else
	{
		QDir dir(m_preferences->getIncomingPath());
		url = QString("file:") + dir.path() + "/" + dir.entryList().grep(((ApollonTransferViewItem *) m_downList->currentItem())->name())[0];
		m_playerTab->play(url, ((ApollonTransferViewItem *) m_downList->currentItem())->name());
	}
}

void ApollonTransferTab::autoSourceSearch()
{
	sourceSearch(true);
}

void ApollonTransferTab::sourceSearch(bool autoSearch)
{
	ApollonTransferViewItem* item = (ApollonTransferViewItem*) m_downList->firstChild();
	ApollonTransferViewItem* sourcesearchitem;
	
	while(item)
	{
		sourcesearchitem = item;
		if((autoSearch || sourcesearchitem->isSelected()) &&
		   sourcesearchitem->status() != i18n("Canceled") && sourcesearchitem->status() != i18n("Completed"))
		{
			int ID = m_gcn->getNewID();

			m_sourceSearchIDs.append(ID);
			
			//if (item->protocolName() != "OpenNap" && item->protocolName() != "Gnutella") 		  	  
			  m_gcn->makeLocateRequest(ID, sourcesearchitem->hash());
		}
		item = item->nextSibling();
	}
}

void ApollonTransferTab::itemReply(QString line)
{
	line = line.stripWhiteSpace();

	int receivedItemID = giFTConnection::searchKey("ITEM", line).toInt();

	if(m_sourceSearchIDs.contains(receivedItemID) > 0)
	{
		// Should there be a test here to see if this reply is already in the list ?

		if(line.find("file(") > -1)
		{
			QString user = giFTConnection::searchKey("user", line);
			QString hash = giFTConnection::searchKey("hash", line);
			QString size = giFTConnection::searchKey("size", line);
			QString url =  giFTConnection::searchKey("url",  line);

			QString temp = giFTConnection::searchKey("file", line);
			QString save = temp.right(temp.length() - temp.findRev("/") - 1);

			m_gcn->makeAddRequest(user, hash, size, url, save);
		}
		else
		{
			QString endsearch = "ITEM(" + QString::number(receivedItemID) + ");";

			if(QString::compare(endsearch, line) == 0)
			{
				m_sourceSearchIDs.remove(receivedItemID);
			}
		}
	}
}

void ApollonTransferTab::sltBrowseUser()
{
	emit browseUser(m_userIP);
}

void ApollonTransferTab::sltGetUserIP(int id)
{
	m_userIP = m_browseUserMenu->text(id);
}

void ApollonTransferTab::countActives()
{
	// sends a signal to the systemtray, which displays the number of downloads in the QToolTip.
	int actives = 0;
	float sumrate = 0;
	QString s;
	ApollonTransferViewItem *item = (ApollonTransferViewItem *) m_downList->firstChild();
	while(item)
	{
		s = item->bandwidth();
		if(s.find("KB/s") != -1)
		{
			actives++;
			sumrate += (s.remove(" KB/s")).toFloat();
		}
		item = item->nextSibling();
	}
	emit statsUpdate(actives, sumrate);
}

void ApollonTransferTab::setTray(ApollonSystemTrayGnome *tray)
{
	m_systemTrayGnome = tray;
}

void ApollonTransferTab::setTray(ApollonSystemTray *tray)
{
	m_systemTray = tray;
}

void ApollonTransferTab::setShowPopup(bool show)
{
	m_showPopup = show;
}

void ApollonTransferTab::sltAutoSearchInterval(int i)
{
	m_autoSearchInterval = i;
	m_sourceSearchTimer = new QTimer;
	m_sourceSearchTimer->start(m_autoSearchInterval * 60000); // m_autoSearchInterval is in minutes
	connect(m_sourceSearchTimer, SIGNAL(timeout()), this, SLOT(autoSourceSearch()));
}

void ApollonTransferTab::contextMenuEvent(QContextMenuEvent *e)
{
	e->consume();
}

QString ApollonTransferTab::protocolName(QString line)
{
	QString protocolName = giFTConnection::searchKey("url", line);

	if(protocolName == "")
		protocolName = giFTConnection::searchKey("hash", line);

	if(protocolName.isEmpty() || protocolName.isNull())
		protocolName = " ";

	protocolName = protocolName.section(":", 0, 0);

	if(protocolName == "FTH")
		protocolName = "FastTrack";

	return protocolName;
}

void ApollonTransferTab::sltCheckWM(bool isKDE){

	m_isWM_KDE = isKDE;
}

