/***************************************************************************
                          preferences.ui.h -  description
                             -------------------
begin		: June  2003
copyright	: (C) 2003 by Martin Koebele
email		: apollon@mkoebele.de
description	: the new apollon-settingsdialog

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <libgift/libgift.h>
#include <libgift/conf.h>

#include <qstring.h>
#include <qstringlist.h> 
#include <qurl.h>

#include <kconfig.h>
#include <kfiledialog.h>
#include <kmessagebox.h>

#include "pluginlistitem.h"


void ApollonPreferencesDialog::init(){
  loader = KGlobal::iconLoader();
  sharedFolderPB->setPixmap(loader->loadIcon("folder", KIcon::Small, 16) );
  disabledPlugins = "";

  noPix = loader->loadIcon("no", KIcon::Small);
  okPix = loader->loadIcon("ok", KIcon::Small);
  
  connect(pluginsList, SIGNAL(clicked(QListBoxItem*)), this, SLOT(sltEnableButton(QListBoxItem*)));
 
}

void ApollonPreferencesDialog::accept()
{
	emit sigOK();
	QDialog::accept();
}

void ApollonPreferencesDialog::uploadsCheckToggled(bool enabled)
{
        uploadsSpin->setEnabled(enabled);
	userUploadsSpin->setEnabled(enabled);
	if (bandwidthCheck->isChecked())
	  upRestrictionSpin->setEnabled(enabled);
}

void ApollonPreferencesDialog::bandwidthCheckToggled(bool enabled)
{
	downRestrictionSpin->setEnabled(enabled);
	if (uploadsCheck->isChecked())
	  upRestrictionSpin->setEnabled(enabled);
	else
	  upRestrictionSpin->setDisabled(TRUE);
}


void ApollonPreferencesDialog::readValues()
{

  QString home(getenv("HOME"));
  QString filename(home);
  filename.append("/.giFT/giftd.conf");
  Config* configuration = config_new_ex ((char*) filename.ascii(), FALSE);

  followSymlinksCheck->setChecked(config_get_int(configuration, (char*) "main/follow_symlinks"));

  QString plugins = config_get_str(configuration, (char*) "main/plugins");

  QStringList pluginslist = pluginslist.split(QString(":"), plugins);
  
  for ( QStringList::Iterator it = pluginslist.begin(); it != pluginslist.end(); ++it ) {
    /*PluginListItem* newItem =*/ new PluginListItem(pluginsList, okPix, *it);
  }
  
 
  mCompleted = config_get_str(configuration, (char*) "download/completed");
  downloadEdit->setText(mCompleted);
  emit sigSharedFolder(mCompleted);

  mIncoming = config_get_str(configuration, (char*) "download/incoming");
  
  
  userUploadsSpin->setValue(config_get_int(configuration, (char*) "sharing/max_peruser_uploads"));
  
  hideHiddenCheck->setChecked(config_get_int(configuration, (char*) "sharing/hide_dot_files"));

  QString shared = config_get_str(configuration, (char*) "sharing/root");
  QStringList sharedList = sharedList.split(QString(":"), shared);
  for ( QStringList::Iterator it = sharedList.begin(); it != sharedList.end(); ++it ) {
    sharedFolderList->insertItem(*it);
    if (!(*it).isEmpty())
    	emit sigSharedFolder(*it);
  }

  int temp = config_get_int(configuration, (char*) "sharing/max_uploads");
  uploadsSpin->setValue(temp);
  if (temp!=0)
    uploadsCheck->setChecked(TRUE);
  else
    uploadsCheck->setChecked(FALSE);
  
  int interval = config_get_int(configuration, (char*) "sharing/auto_resync_interval");
  if (interval == 0)
    syncCombo->setCurrentItem(0);
  else if (interval == 3600)
    syncCombo->setCurrentItem(1);
  else if (interval == 86400)
    syncCombo->setCurrentItem(2);
  else
    syncCombo->setCurrentItem(3);

  
  int shareCompleted = config_get_int(configuration, (char*) "sharing/share_completed");
  shareDownloadCheck->setChecked(shareCompleted);


  int downstream = config_get_int(configuration, (char*) "bandwidth/downstream")/1024;  //B/s->KB/s
  int upstream = config_get_int(configuration, (char*) "bandwidth/upstream")/1024;
  
  if (downstream || upstream) {
    bandwidthCheck->setChecked(TRUE);
    downRestrictionSpin->setValue(downstream);
    upRestrictionSpin->setValue(upstream);
  }
  else
    bandwidthCheck->setChecked(FALSE);
  
  config_free(configuration);

   
   //username

  QString filename2(home);
  filename2.append("/.giFT/OpenFT/OpenFT.conf");
  Config* openFTConfig = config_new_ex ((char*)filename2.ascii(), FALSE);
  userEdit->setText(config_get_str(openFTConfig, (char*) "main/alias"));

  int searchnode = config_get_int(openFTConfig, (char*)"main/class");
  
  if (searchnode == 1) 
  	checkSearchnode->setChecked(false);
  else if (searchnode == 3) {
  	checkSearchnode->setChecked(true);
	groupBoxPorts->setEnabled(true);
	}
  
   spinSearchPort->setValue(config_get_int(openFTConfig, (char*)"main/port"));
   spinHttpPort->setValue(config_get_int(openFTConfig, (char*)"main/port_http"));
  config_free(openFTConfig);

}


 
void ApollonPreferencesDialog::readConfig() {
 
  readValues();

  KConfig *config = KGlobal::config();
  { 
    KConfigGroupSaver saver(config, "Behaviour");
    
    bool stop = config->readBoolEntry("shutApollon", false);
    removeSysTrayCheck->setChecked(stop);
    emit sigShutApollon(stop);
				  
    bool mStartDocked = config->readBoolEntry("Docked", false);
    startSysTrayCheck->setChecked(mStartDocked);
    emit sigStartDocked(mStartDocked);

    bool pops = config->readBoolEntry("ShowPopups", true);
    notificationCheck->setChecked(pops);
    emit sigShowPopUps(pops);

    int startpage = config->readNumEntry("StartPage", 0);
    pageCombo->setCurrentItem (startpage);
    emit sigStartPage(startpage);

    int killdaemon= config->readBoolEntry("KillDaemon", true);
    killDaemonCheck->setChecked(!killdaemon);
    emit sigKillDaemon(killdaemon);

    QStringList disabledPluginList = config->readListEntry("disabled_plugins",':');
    for ( QStringList::Iterator it = disabledPluginList.begin(); it != disabledPluginList.end(); ++it ) {
      PluginListItem* newItem = new PluginListItem(pluginsList, noPix, *it);
      newItem->setEnabled(FALSE);
    }

    {
      KConfigGroupSaver saver(config, "giFT");
      int autosearchinterval = config->readNumEntry("AutoSearchInterval", 30);
      autoSearchSpin->setValue(autosearchinterval);
      emit sigAutoSearchInterval(autosearchinterval);

      int max_uploads = config->readNumEntry("max_uploads", 5);
      int max_per_user = config->readNumEntry("max_per_user", 1);
      if (!(uploadsCheck->isChecked())) {
	uploadsSpin->setValue(max_uploads);
	userUploadsSpin->setValue(max_per_user);
      }

      int downstream = config->readNumEntry("downstream", 10*1024)/1024;
      int upstream = config->readNumEntry("upstream", 5*1024)/1024;
      if (!(bandwidthCheck->isChecked())){
	downRestrictionSpin->setValue(downstream);
	upRestrictionSpin->setValue(upstream);
      }
    }
    pluginsList->sort(FALSE);
  }
  mGiftChanged = FALSE;
}

void ApollonPreferencesDialog::saveConfig() {//this slot gets called, when the settings are changed, therefore
  // we also have to re-send the informations

  
  KConfig *config = KGlobal::config();
  { 
    KConfigGroupSaver saver(config, "Behaviour");

    config->writeEntry("shutApollon", removeSysTrayCheck->isChecked());
    emit sigShutApollon(removeSysTrayCheck->isChecked());

    config->writeEntry("Docked", startSysTrayCheck->isChecked());
    
    config->writeEntry("ShowPopups", notificationCheck->isChecked());
    emit sigShowPopUps(notificationCheck->isChecked());

    config->writeEntry("StartPage", pageCombo->currentItem());
   
    config->writeEntry("KillDaemon", !killDaemonCheck->isChecked());
    emit sigKillDaemon(!killDaemonCheck->isChecked());
    
    disabledPlugins = QString("");
    for (unsigned int i = 0; i < pluginsList->count(); ++i) {
      if (!(((PluginListItem*)pluginsList->item(i))->isEnabled())){
	disabledPlugins += ((PluginListItem*)pluginsList->item(i))->text();
	if ((i+1) < pluginsList->count())
	  disabledPlugins += QString(":");
      }
    }
   
    config->writeEntry("disabled_plugins", disabledPlugins);
  }
  {
    KConfigGroupSaver saver(config, "giFT");
    
    config->writeEntry("AutoSearchInterval", autoSearchSpin->value());
    emit sigAutoSearchInterval(autoSearchSpin->value());
    
    config->writeEntry("max_uploads", uploadsSpin->value());
    config->writeEntry("max_per_user", userUploadsSpin->value());
    config->writeEntry("downstream", downRestrictionSpin->value()*1024);
    config->writeEntry("upstream", upRestrictionSpin->value()*1024);
    
  }
  
  
  config->sync(); //write data immediately to disk
  
  saveValues();
  
  if (mGiftChanged){
    int answer = KMessageBox::questionYesNo(this, tr2i18n("You made some changes which can only be applied after reconnecting.\nDo you want to reconnect now?"), tr2i18n("Settings changes"));
    if (answer == KMessageBox::Yes)
      emit sigRestartDaemon();
    mGiftChanged = FALSE;
  }
  
  
}



void ApollonPreferencesDialog::sltDownloadDir()
{
  KURL url = dialog.selectDirectory(downloadEdit->text());
  if(!url.isEmpty())
    downloadEdit->setText(url.path());
}


void ApollonPreferencesDialog::sltAddSharedFolder()
{
  KURL url = dialog.selectDirectory();
  if(!url.isEmpty())
    sharedFolderList->insertItem(url.path());
}


void ApollonPreferencesDialog::deleteSharedFolder()
{
    sharedFolderList->removeItem(sharedFolderList->currentItem());
}


void ApollonPreferencesDialog::saveValues()
{
  QString home(getenv("HOME"));
  QString filename(home);
  filename.append("/.giFT/giftd.conf");
  Config* configuration = config_new_ex ((char*) filename.ascii(), FALSE);


  config_set_int(configuration, (char*) "main/follow_symlinks", followSymlinksCheck->isChecked());

  QString plugins("");

  for (unsigned int i = 0; i < pluginsList->count(); ++i) {
    if (((PluginListItem*)pluginsList->item(i))->isEnabled()){
      plugins += ((PluginListItem*)pluginsList->item(i))->text();
      if ((i+1) < pluginsList->count())
	plugins += QString(":");
    }
  }


  config_set_str(configuration, (char*) "main/plugins", (char*) plugins.ascii());

  config_set_str(configuration, (char*) "download/completed", (char*) downloadEdit->text().ascii());

  config_set_int(configuration, (char*) "sharing/max_peruser_uploads",userUploadsSpin->value());

  config_set_int(configuration, (char*) "sharing/hide_dot_files",   hideHiddenCheck->isChecked());


  QString shared("");
  for (unsigned int i = 0; i < sharedFolderList->count(); ++i) {
    shared += sharedFolderList->text(i);
    if ((i+1) < sharedFolderList->count())
      shared += QString(":");
  }

  config_set_str(configuration, (char*) "sharing/root", (char*) shared.ascii());

  if (uploadsCheck->isChecked())
    config_set_int(configuration, (char*) "sharing/max_uploads", uploadsSpin->value());
  else
    config_set_int(configuration, (char*) "sharing/max_uploads", 0);

  int interval;
  int sync = syncCombo->currentItem();

  if (sync == 0)
    interval = 0;
  else if (sync == 1)
    interval = 3600;
  else if (sync == 2)
    interval = 86400;
  else
    interval = 604800;
  config_set_int(configuration, (char*) "sharing/auto_resync_interval", interval);


  config_set_int(configuration, (char*) "sharing/share_completed", shareDownloadCheck->isChecked());


  int downstream = 0, upstream = 0;
  if (bandwidthCheck->isChecked()){
    downstream = downRestrictionSpin->value();
    upstream = upRestrictionSpin->value();
  }
  config_set_int(configuration, (char*) "bandwidth/downstream", downstream*1024);
  config_set_int(configuration, (char*) "bandwidth/upstream", upstream*1024);


  config_write(configuration);


  config_free(configuration);


  //username
  filename = home + "/.giFT/OpenFT/OpenFT.conf";
  if(QFile::exists(filename))
  {
    configuration = config_new_ex((char *) filename.ascii(), false);
    config_set_str(configuration, (char *) "main/alias", (char *) userEdit->text().ascii());
    config_write(configuration);
    config_free(configuration);
  }
  filename = home + "/.giFT/OpenFT/OpenFT.conf";
  if(QFile::exists(filename))
  {
    configuration = config_new_ex((char *) filename.ascii(), false);
    config_set_str(configuration, (char *) "main/alias", (char *) userEdit->text().ascii());
    
    config_set_int(configuration, (char *) "main/class", checkSearchnode->isChecked() ? 3 : 1);
    config_set_int(configuration, (char *) "main/port", spinSearchPort->value());
    config_set_int(configuration, (char *) "main/port_http", spinHttpPort->value());
    
    config_write(configuration);
    config_free(configuration);
  }
}

void ApollonPreferencesDialog::sltNewPlugin()
{
  QString path = KFileDialog::getOpenFileName("/usr/local/lib/giFT","*.so");
  if(!path.isEmpty()){
    QUrl url = QUrl(QString("file:") + path);
    QString file = url.fileName();  
    if (file.contains("lib")){
      path =  file.remove("lib");
      path = path.remove(".so");
    }
    /*PluginListItem* newIt = */new PluginListItem(pluginsList, okPix, path);
  }
}


void ApollonPreferencesDialog::sltDeletePlugin()
{
  pluginsList->removeItem(pluginsList->currentItem());
}



void ApollonPreferencesDialog::sltDisablePlugin()
{
  int i = pluginsList->currentItem();
  PluginListItem* item = (PluginListItem*)pluginsList->item(i);
  QString text = item->text();
  
  pluginsList->removeItem(i);


  
  if (item->isEnabled()){
    enablePluginButton->setText(tr2i18n("&Enable"));
    PluginListItem* newItem = new PluginListItem(pluginsList, noPix, text);
    newItem->setEnabled(FALSE);

  }
  else{
    enablePluginButton->setText(tr2i18n("D&isable"));
    PluginListItem* newItem = new PluginListItem(pluginsList, okPix, text);
    newItem->setEnabled(TRUE);
    
  }
  pluginsList->sort(FALSE);
}


void ApollonPreferencesDialog::sltEnableButton( QListBoxItem* it)
{
  if (it) {
  PluginListItem* item = (PluginListItem*) it;
  if (item->isEnabled())
    enablePluginButton->setText(tr2i18n("D&isable"));
  else
    enablePluginButton->setText(tr2i18n("&Enable"));
  }
}


QString ApollonPreferencesDialog::getIncomingPath() {
  QString home(getenv("HOME"));
  mIncoming = mIncoming.replace(QString("~"), home);
  return mIncoming;
}

QString ApollonPreferencesDialog::getCompletedPath() {
  QString home(getenv("HOME"));
  mCompleted = mCompleted.replace(QString("~"), home);
  return mCompleted;
}


void ApollonPreferencesDialog::sltGiftSettingsChanged()
{
    mGiftChanged = TRUE;
}
